@@ -22,8 +22,6 @@ from webob import exc
 from nova.api.openstack.compute import block_device_mapping_v1 \
         as block_device_mapping
 from nova.api.openstack.compute import extension_info
-from nova.api.openstack.compute.legacy_v2 import extensions
-from nova.api.openstack.compute.legacy_v2 import servers as servers_v2
 from nova.api.openstack.compute import servers as servers_v21
 from nova.compute import api as compute_api
 from nova import db
@@ -31,6 +29,7 @@ from nova import exception
 from nova import test
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit.image import fake
+from nova.tests import uuidsentinel as uuids
 
 CONF = cfg.CONF
 
@@ -118,7 +117,7 @@ class BlockDeviceMappingTestV21(test.TestCase):
         self.mox.StubOutWithMock(compute_api.API, '_validate_bdm')
         self.mox.StubOutWithMock(compute_api.API, '_get_bdm_image_metadata')
         volume = {
-            'id': 1,
+            'id': uuids.volume_id,
             'status': 'active',
             'volume_image_metadata':
                 {'test_key': 'test_value'}
@@ -141,6 +140,51 @@ class BlockDeviceMappingTestV21(test.TestCase):
         self.mox.ReplayAll()
         self._test_create(params, no_image=True)
 
+    @mock.patch.object(compute_api.API, '_validate_bdm')
+    @mock.patch.object(compute_api.API, '_get_bdm_image_metadata')
+    def test_create_instance_with_imageRef_as_empty_string(
+        self, mock_bdm_image_metadata, mock_validate_bdm):
+        volume = {
+            'id': uuids.volume_id,
+            'status': 'active',
+            'volume_image_metadata':
+                {'test_key': 'test_value'}
+        }
+        mock_bdm_image_metadata.return_value = volume
+        mock_validate_bdm.return_value = True
+        params = {'block_device_mapping': self.bdm,
+                  'imageRef': ''}
+        old_create = compute_api.API.create
+
+        def create(*args, **kwargs):
+            self.assertEqual(kwargs['block_device_mapping'], self.bdm)
+            return old_create(*args, **kwargs)
+
+        self.stub_out('nova.compute.api.API.create', create)
+        self._test_create(params)
+
+    def test_create_instance_with_imageRef_as_full_url(self):
+        bdm = [{
+            'volume_id': self.volume_id,
+            'device_name': 'vda'
+        }]
+        image_href = ('http://localhost/v2/fake/images/'
+                      '76fa36fc-c930-4bf3-8c8a-ea2a2420deb6')
+        params = {'block_device_mapping': bdm,
+                  'imageRef': image_href}
+        self.assertRaises(exception.ValidationError,
+                          self._test_create, params)
+
+    def test_create_instance_with_non_uuid_imageRef(self):
+        bdm = [{
+            'volume_id': self.volume_id,
+            'device_name': 'vda'
+        }]
+        params = {'block_device_mapping': bdm,
+                  'imageRef': 'bad-format'}
+        self.assertRaises(exception.ValidationError,
+                          self._test_create, params)
+
     def test_create_instance_with_volumes_disabled(self):
         bdm = [{'device_name': 'foo'}]
         params = {'block_device_mapping': bdm}
@@ -361,74 +405,3 @@ class BlockDeviceMappingTestV21(test.TestCase):
                   'block_device_mapping_v2': bdm_v2}
         self.assertRaises(exc.HTTPBadRequest, self._test_create, params,
                           override_controller=both_controllers)
-
-
-class BlockDeviceMappingTestV2(BlockDeviceMappingTestV21):
-    validation_error = exc.HTTPBadRequest
-
-    def _setup_controller(self):
-        self.ext_mgr = extensions.ExtensionManager()
-        self.ext_mgr.extensions = {'os-volumes': 'fake'}
-        self.controller = servers_v2.Controller(self.ext_mgr)
-        self.ext_mgr_no_vols = extensions.ExtensionManager()
-        self.ext_mgr_no_vols.extensions = {}
-        self.no_volumes_controller = servers_v2.Controller(
-            self.ext_mgr_no_vols)
-
-    def test_create_instance_with_volumes_disabled(self):
-        bdm = [{'device_name': 'foo'}]
-        params = {'block_device_mapping': bdm}
-        old_create = compute_api.API.create
-
-        def create(*args, **kwargs):
-            self.assertIsNone(kwargs['block_device_mapping'])
-            return old_create(*args, **kwargs)
-
-        self.stubs.Set(compute_api.API, 'create', create)
-        self._test_create(params,
-                          override_controller=self.no_volumes_controller)
-
-    def test_create_instance_decide_format_legacy(self):
-        ext_mgr = extensions.ExtensionManager()
-        ext_mgr.extensions = {'os-volumes': 'fake',
-                              'os-block-device-mapping-v2-boot': 'fake'}
-        controller = servers_v2.Controller(self.ext_mgr)
-        bdm = [{'device_name': 'foo1',
-                'volume_id': fakes.FAKE_UUID,
-                'delete_on_termination': 1}]
-
-        expected_legacy_flag = True
-
-        old_create = compute_api.API.create
-
-        def create(*args, **kwargs):
-            legacy_bdm = kwargs.get('legacy_bdm', True)
-            self.assertEqual(legacy_bdm, expected_legacy_flag)
-            return old_create(*args, **kwargs)
-
-        def _validate_bdm(*args, **kwargs):
-            pass
-
-        self.stubs.Set(compute_api.API, 'create', create)
-        self.stubs.Set(compute_api.API, '_validate_bdm',
-                       _validate_bdm)
-
-        self._test_create({}, override_controller=controller)
-
-        params = {'block_device_mapping': bdm}
-        self._test_create(params, override_controller=controller)
-
-    def test_create_instance_with_size_empty_string(self):
-        # Add a check whether the size is an empty string
-        # in V2.1 API only. So this test is skipped in V2.0 API
-        pass
-
-    def test_create_instance_with_size_zero(self):
-        # Add a check whether the size is zero in V2.1 API only.
-        # So this test is skipped in V2.0 API
-        pass
-
-    def test_create_instance_with_size_greater_than_limit(self):
-        # Add a check whether size is greater than the limit
-        # in V2.1 API only. So this test is skipped in V2.0 API
-        pass
