@@ -228,20 +228,29 @@ class Guest(object):
 
         return interfaces
 
-    def get_interface_by_mac(self, mac):
-        """Lookup a LibvirtConfigGuestInterface by the MAC address.
+    def get_interface_by_cfg(self, cfg):
+        """Lookup a full LibvirtConfigGuestInterface with
+        LibvirtConfigGuestInterface generated
+        by nova.virt.libvirt.vif.get_config.
 
-        :param mac: MAC address of the guest interface.
-        :type mac: str
+        :param cfg: config object that represents the guest interface.
+        :type cfg: LibvirtConfigGuestInterface object
         :returns: nova.virt.libvirt.config.LibvirtConfigGuestInterface instance
             if found, else None
         """
 
-        if mac:
+        if cfg:
             interfaces = self.get_all_devices(
                 vconfig.LibvirtConfigGuestInterface)
             for interface in interfaces:
-                if interface.mac_addr == mac:
+                # NOTE(leehom) LibvirtConfigGuestInterface get from domain and
+                # LibvirtConfigGuestInterface generated by
+                # nova.virt.libvirt.vif.get_config must be identical.
+                if (interface.mac_addr == cfg.mac_addr and
+                        interface.net_type == cfg.net_type and
+                        interface.source_dev == cfg.source_dev and
+                        interface.target_dev == cfg.target_dev and
+                        interface.vhostuser_path == cfg.vhostuser_path):
                     return interface
 
     def get_vcpus_info(self):
@@ -348,10 +357,10 @@ class Guest(object):
                 devs.append(dev)
         return devs
 
-    def detach_device_with_retry(self, get_device_conf_func, device,
-                                 persistent, live, max_retry_count=7,
-                                 inc_sleep_time=2,
-                                 max_sleep_time=30):
+    def detach_device_with_retry(self, get_device_conf_func, device, live,
+                                 max_retry_count=7, inc_sleep_time=2,
+                                 max_sleep_time=30,
+                                 alternative_device_name=None):
         """Detaches a device from the guest. After the initial detach request,
         a function is returned which can be used to ensure the device is
         successfully removed from the guest domain (retrying the removal as
@@ -360,8 +369,6 @@ class Guest(object):
         :param get_device_conf_func: function which takes device as a parameter
                                      and returns the configuration for device
         :param device: device to detach
-        :param persistent: bool to indicate whether the change is
-                           persistent or not
         :param live: bool to indicate whether it affects the guest in running
                      state
         :param max_retry_count: number of times the returned function will
@@ -372,11 +379,18 @@ class Guest(object):
                                time will not be incremented using param
                                inc_sleep_time. On reaching this threshold,
                                max_sleep_time will be used as the sleep time.
+        :param alternative_device_name: This is an alternative identifier for
+            the device if device is not an ID, used solely for error messages.
         """
+        alternative_device_name = alternative_device_name or device
+
         def _try_detach_device(conf, persistent=False, live=False):
             # Raise DeviceNotFound if the device isn't found during detach
             try:
                 self.detach_device(conf, persistent=persistent, live=live)
+                LOG.debug('Successfully detached device %s from guest. '
+                          'Persistent? %s. Live? %s',
+                          device, persistent, live)
             except libvirt.libvirtError as ex:
                 with excutils.save_and_reraise_exception():
                     errcode = ex.get_error_code()
@@ -385,19 +399,40 @@ class Guest(object):
                         if 'not found' in errmsg:
                             # This will be raised if the live domain
                             # detach fails because the device is not found
-                            raise exception.DeviceNotFound(device=device)
+                            raise exception.DeviceNotFound(
+                                device=alternative_device_name)
                     elif errcode == libvirt.VIR_ERR_INVALID_ARG:
                         errmsg = ex.get_error_message()
                         if 'no target device' in errmsg:
                             # This will be raised if the persistent domain
                             # detach fails because the device is not found
-                            raise exception.DeviceNotFound(device=device)
+                            raise exception.DeviceNotFound(
+                                device=alternative_device_name)
 
         conf = get_device_conf_func(device)
         if conf is None:
-            raise exception.DeviceNotFound(device=device)
+            raise exception.DeviceNotFound(device=alternative_device_name)
 
-        _try_detach_device(conf, persistent, live)
+        persistent = self.has_persistent_configuration()
+
+        LOG.debug('Attempting initial detach for device %s',
+                  alternative_device_name)
+        try:
+            _try_detach_device(conf, persistent, live)
+        except exception.DeviceNotFound:
+            # NOTE(melwitt): There are effectively two configs for an instance.
+            # The persistent config (affects instance upon next boot) and the
+            # live config (affects running instance). When we detach a device,
+            # we need to detach it from both configs if the instance has a
+            # persistent config and a live config. If we tried to detach the
+            # device with persistent=True and live=True and it was not found,
+            # we should still try to detach from the live config, so continue.
+            if persistent and live:
+                pass
+            else:
+                raise
+        LOG.debug('Start retrying detach until device %s is gone.',
+                  alternative_device_name)
 
         @loopingcall.RetryDecorator(max_retry_count=max_retry_count,
                                     inc_sleep_time=inc_sleep_time,
@@ -411,8 +446,8 @@ class Guest(object):
                 _try_detach_device(config, persistent=False, live=live)
 
                 reason = _("Unable to detach from guest transient domain.")
-                raise exception.DeviceDetachFailed(device=device,
-                                                   reason=reason)
+                raise exception.DeviceDetachFailed(
+                    device=alternative_device_name, reason=reason)
 
         return _do_wait_and_retry_detach
 
